# Simple dart web widgets

Библиотека максимально простых Web-компонент на дарт. 
Не совместимо с Flutter.

Читать на других языках: [English](README.md), [Русский](README.ru.md).

Живой пример использования: [simple_dart_web_showcase](https://viktorzahar.github.io/simple_dart_web_showcase)

##Главные принципы библиотеки

- Компоненты должны быть простыми, гибкими и подходящими для кастомизации в каждом отдельном проекте.
- ООП подход
- Отсутствие магии (без генерации кода, без реактивщины, без DI итд).

## Разделение компонентов на тему и логику
- Одна тема определяется в одном CSS файле.
- Системные стили никак не связанные с темами выделены в файл system_style.css.

### Свойства компонента определяемые в теме (через CSS)
- font
- background
- color
- border
- font
- images
- padding (для компонентов)

### Свойства компонента определяемые в коде
- stride
- width
- height
- vertical
- wrap
- align
- scrollable
- fillContent
- padding (для панелей)

### Конвенции в названиях
- Название всех CSS-классов начинаются с заглавной буквы и соответствуют названию классов в Dart.

## Композиция компонентов
- В библиотеке существует всего один компонент для композиции - класс Panel. С помощью комбинации панелей можно создавать любые лэйауты.
- Panel - не предназначен чтобы от него наследоваться. 
- Для создания новых компонентов на основе панели используйте абстрактный класс PanelComponent. В качестве параметра нужно передать название нового dart-класса.

Например:

    class SomeNewComplexComponent extends PanelComponent {
        SomeNewComplexComponent(): super('SomeNewComplexComponent') {
            vertical = true;
            ...
        }
    }

- Простые компоненты которые не нуждаются в композиции - наследуются от класса Component. Также в качестве параметра нужно передать название нового dart-класса.

Например:

    class SomeNewComponent extends Component {
        SomeNewComponent(): super('SomeNewComponent') {

        }

        @override
        Element nodeRoot = DivElement();
    }

Все компоненты имеют nodeRoot - который помещается в DOM документа.
Можно доступаться до стиля компонентов с помощью nodeRoot.style - но только в экстренных случаях или в процессе разработки.

### Вспомогательные CSS-классы

- Active - выделяет активное состояние компонента (например текущая вкладка или зажатая кнопка).
- Disabled - выделяет отключенное состояние компонента.

Эти классы используются только в миксинах MixinActivate и MixinDisable - если нужно добавить активируемость или запрет на редактирование в ваши компоненты просто подмешайте эти миксины в ваш новый компонент.

### Примеры использования библиотеки:
Для проверки внешнего вида тем можно использовать проект: [simple_dart_web_showcase](https://github.com/ViktorZahar/simple_dart_web_showcase)

Для создания single page applications рекомендуется использовать библиотеку: [simple_dart_web_views](https://github.com/ViktorZahar/simple_dart_web_views)